
/*********************************************************

INCLUDE FILE: communications.agc
  by Steve Vink, April 2015
  Use permitted for all purposes, commercial and personal

	This file contains functions to facilitate multi-player projects

// Project: Communications
// Created: 2015-04-25


**********************************************************/

// Constants - these make the code easier to read
#constant cHOST = 1
#constant cCLIENT = 2

// Network Type to hold all of our variables
type tNetwork
	networkID
	Status
	messageCount
endtype

global gNetwork as tNetwork
global arrNetworkMessage as string[100]

global gMsg as string

//*********************************************************
function CommsCreateHost()
//*********************************************************
//	Sets the application to be the host
//
//	In this module, the network ame, host name and port numbers
//  are hard-coded for simplicity
//  In a more complex environment you may want to  be more
//	dynamic with these settings.
//
//********************************************************* 

	gNetwork.networkID = HostNetwork("AGK_NETWORK", "HOST", 45654)
	
	// Wait up to 3 seconds to create the host
	// If no netwrok created after 3 seconds, return a success of 0
	time# = timer() + 3
	while time# > timer()
		if IsNetworkActive(gNetwork.networkID)
			exitfunction 1
		endif
	endwhile
	
	message("Host Creation Failed")
	
endfunction 0


//*********************************************************
function CommsJoinNetwork()
//*********************************************************
//	Sets the application to be a client
//
//  The host must have already created the network or it will fail
//*********************************************************

	gNetwork.networkID = JoinNetwork("AGK_NETWORK", "CLIENT")
	
	// Wait up to 3 seconds to join the network
	// If no network joined after 3 seconds, return a success of 0
	// We know that the connection has succeeded when we have > 1 client
	// (This client + the host)
	time# = timer() + 3
	while time# > timer()
		if GetNetworkNumClients(gNetwork.networkID) > 1
			exitfunction 1
		endif
	endwhile

	message("Network Connection Failed")
	
endfunction 0


//*********************************************************
function CommsSendMessage(msgString$)
//*********************************************************
//	Sends a message to the host and other clients
//
//	In this approach, we simply send a single string.
//	This string may have several delimited parameters, e.g...
//
//	CHAR:MyName:20:1.5:Hello Everybody
// 
//	The receiving client/host is responsible for parsing and
//	interpreting the message
//*********************************************************

	msg = CreateNetworkMessage()
	AddNetworkMessageString(msg,msgString$)
	SendNetworkMessage(gNetwork.networkID,0,msg)

endfunction

//*********************************************************
function CommsReceiveMessage()
//*********************************************************
//	Checks for new Messages and receives them
//
//	We will simply receive the message and store it.
//	It is the responsibility of the main program to 
//	take the messages and process them accordingly.
//*********************************************************

	// initialise a counter to track message number
	gNetwork.messageCount = 0
	
	do
		msg = GetNetworkMessage(gNetwork.networkID)
		if msg = 0
			exit
		endif
			
		inc gNetwork.messageCount, 1
		arrNetworkMessage[gNetwork.messageCount] = GetNetworkMessageString(msg)
		
		DeleteNetworkMessage(msg)
		
	loop
	
endfunction

//*********************************************************
function CommsMessageCount()
//*********************************************************
//	Returns the number of messages
// In a self-contained module like this, the main program
// should never directly access the variables,
//	no matter how simple the process.
//*********************************************************


	count = gNetwork.MessageCount
	
endfunction count

//*********************************************************
function CommsGetMessage(id)
//*********************************************************
//	Returns a specific message
//	use CommsMessageCount() to determine ow many messages are available.
//
//	The message is cleared after sending 
//
//*********************************************************

	message$ = arrNetworkMessage[id]
	arrNetworkMessage[id] = ""
	
endfunction message$

//*********************************************************
function CommsIsHost()
//*********************************************************
// Returns true if this device is the host
//*********************************************************

	if gNetwork.Status = cHOST
		exitfunction 1
	endif

endfunction 0
